package org.mai806.catalog.dao.jdbc;

import org.mai806.catalog.model.*;
import org.mai806.catalog.dao.Session;

import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Types;
import java.util.List;
import java.util.ArrayList;

/**
 *      CatalogItem
 */
public class CatalogItemDAO extends BaseDAO {

    public CatalogItemDAO(Session session) {
        super(session);
    }

    private static final String BOOK_ITEM_TYPE = "BOOK";
    private static final String MOVIE_ITEM_TYPE = "MOVIE";

    public List<CatalogItem> getItemsByCatalog(Catalog catalog) throws Exception {
        PreparedStatement stmt = null;
        ResultSet rs = null;
        List<CatalogItem> list = new ArrayList<CatalogItem>();
        try {
            stmt = getConnection().prepareStatement("select ID, CATALOG_ID, AUTHOR_ID, NAME, ISBN, GENRE, ITEM_TYPE " +
                    "from CATALOG_ITEM where CATALOG_ID=?");
            stmt.setLong(1, catalog.getId());
            rs = stmt.executeQuery();
            while (rs.next()) {
                list.add( readCatalogItem(catalog, null, rs) );
            }
        } finally {
            close(stmt, rs);
        }
        return list;
    }

    public List<CatalogItem> getItemsByAuthor(Author author) throws Exception {
        PreparedStatement stmt = null;
        ResultSet rs = null;
        List<CatalogItem> list = new ArrayList<CatalogItem>();
        try {
            stmt = getConnection().prepareStatement("select ID, CATALOG_ID, AUTHOR_ID, NAME, ISBN, GENRE, ITEM_TYPE " +
                    "from CATALOG_ITEM where AUTHOR_ID=?");
            stmt.setLong(1, author.getId());
            rs = stmt.executeQuery();
            while (rs.next()) {
                list.add( readCatalogItem(null, author, rs) );
            }
        } finally {
            close(stmt, rs);
        }
        return list;
    }

    public CatalogItem saveItem(CatalogItem item) throws Exception {
        PreparedStatement stmt = null;
        try {
            if (item.getId()==null) { // 
                long newID = generateID();
                item.setId(newID);
                stmt = getConnection().prepareStatement("insert into CATALOG_ITEM(NAME,CATALOG_ID,AUTHOR_ID,ISBN,GENRE,ITEM_TYPE,ID)"
                        +" values(?,?,?,?,?,?,?)");
            } else { //  
                stmt = getConnection().prepareStatement("update CATALOG set NAME=?, CATALOG_ID=?, "
                        +"AUTHOR_ID=?, ISBN=?, GENRE=?, ITEM_TYPE=? where ID=?");
            }
            stmt.setString(1, item.getName());
            stmt.setLong(2, item.getCatalog().getId());
            if (item.getAuthor()!=null) {
                stmt.setLong(3, item.getAuthor().getId());
                stmt.setString(6, BOOK_ITEM_TYPE);
            } else {
                stmt.setNull(3, Types.INTEGER);
            }
            if (item instanceof BookItem) {
                stmt.setString(4, ((BookItem)item).getISBN());
                stmt.setString(6, MOVIE_ITEM_TYPE);
            } else {
                stmt.setNull(4, Types.VARCHAR);
            }
            if (item instanceof MovieItem) {
                stmt.setString(5, ((MovieItem)item).getGenre().toString());
            } else {
                stmt.setNull(5, Types.VARCHAR);
            }
            stmt.setLong(7, item.getId());
            stmt.execute();
        } finally {
            close(stmt);
        }
        return item;
    }

    private CatalogItem readCatalogItem(Catalog catalog, Author author, ResultSet rs) throws Exception {
        CatalogItem item;
        String type = rs.getString("ITEM_TYPE");
        if (type.equals(BOOK_ITEM_TYPE)) {
            BookItem book = new BookItem();
            book.setISBN(rs.getString("ISBN"));
            item = book;
        } else if (type.equals(MOVIE_ITEM_TYPE)) {
            MovieItem movie = new MovieItem();
            movie.setGenre(
                    MovieItem.MovieGenre.valueOf(rs.getString("GENRE"))
            );
            item = movie;
        } else {
            throw new Exception("Item type not supported");
        }
        item.setId(rs.getLong("ID"));
        item.setName(rs.getString("NAME"));
        item.setCatalog(catalog);
        item.setAuthor(author);
        return item;
    }
}
